@@ -1,6 +1,6 @@
 /*******************************************************************************
  * #-------------------------------------------------------------------------------
- * # Copyright (c) 2003-2016 IPI PAN.
+ * # dmLab 2003-2019
  * # All rights reserved. This program and the accompanying materials
  * # are made available under the terms of the GNU Public License v3.0
  * # which accompanies this distribution, and is available at
@@ -15,13 +15,7 @@
  * # Algorithm 'SLIQ' developed by Mariusz Gromada
  * # R Package developed by Michal Draminski & Julian Zubek
  * #-------------------------------------------------------------------------------
- * # If you want to use dmLab or MCFS/MCFS-ID, please cite the following paper:
- * # M.Draminski, A.Rada-Iglesias, S.Enroth, C.Wadelius, J. Koronacki, J.Komorowski 
- * # "Monte Carlo feature selection for supervised classification", 
- * # BIOINFORMATICS 24(1): 110-117 (2008)
- * #-------------------------------------------------------------------------------
  *******************************************************************************/
-
 package dmLab.classifier;
 
 import dmLab.utils.cmatrix.ConfusionMatrix;
@@ -107,7 +101,7 @@ public class PredictionResult{
 		}else if(type == Classifier.MODEL_CLASSIFIER && confusionMatrix != null){
 			StringBuffer tmp = new StringBuffer();
 			tmp.append(confusionMatrix.toString(true, true, false, "\t"));
-			tmp.append(confusionMatrix.statsToString(4));
+			tmp.append(confusionMatrix.statsToString(4, true));
 			return tmp.toString();
 		}else {
 			return "";
