@@ -1,6 +1,6 @@
 /*******************************************************************************
  * #-------------------------------------------------------------------------------
- * # Copyright (c) 2003-2016 IPI PAN.
+ * # dmLab 2003-2019
  * # All rights reserved. This program and the accompanying materials
  * # are made available under the terms of the GNU Public License v3.0
  * # which accompanies this distribution, and is available at
@@ -15,14 +15,11 @@
  * # Algorithm 'SLIQ' developed by Mariusz Gromada
  * # R Package developed by Michal Draminski & Julian Zubek
  * #-------------------------------------------------------------------------------
- * # If you want to use dmLab or MCFS/MCFS-ID, please cite the following paper:
- * # M.Draminski, A.Rada-Iglesias, S.Enroth, C.Wadelius, J. Koronacki, J.Komorowski 
- * # "Monte Carlo feature selection for supervised classification", 
- * # BIOINFORMATICS 24(1): 110-117 (2008)
- * #-------------------------------------------------------------------------------
  *******************************************************************************/
 package dmLab.experiment.discretization;
 
+import java.io.File;
+
 import dmLab.array.FArray;
 import dmLab.array.functions.DiscFunctions;
 import dmLab.array.loader.File2Array;
@@ -100,7 +97,7 @@ class DiscretizationBody
 		//System.out.println(" ### dicretized array ### ");
 		//System.out.println(inputArray.toString());
 			
-		array2File.saveFile(inputArray,discParams.resFilesPATH+discParams.outputFileName);
+		array2File.saveFile(inputArray,discParams.resFilesPATH+File.separator+discParams.outputFileName);
 		
 		if(discParams.verbose) 
 			System.out.println("Discretizing have been done!");
